# -*- coding: utf-8 -*-


class BColors:
    HEADERD = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNINGD = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

    @staticmethod
    def __handle(message, color, log):
        tolog = color + message + BColors.ENDC
        if log:
            print(tolog)
        else:
            return tolog

    @staticmethod
    def success(message, log=True):
        return BColors.__handle(message, BColors.OKGREEN + BColors.BOLD, log)

    @staticmethod
    def info(message, log=True):
        return BColors.__handle(message, BColors.OKBLUE, log)

    @staticmethod
    def error(message, log=True):
        return BColors.__handle(message, BColors.FAIL + BColors.BOLD, log)

    @staticmethod
    def header(message, log=True):
        return BColors.__handle(message, BColors.HEADERD, log)

    @staticmethod
    def warning(message, log=True):
        return BColors.__handle(message, BColors.WARNINGD, log)



# -*- coding: utf-8 -*-

from ..model.config import AlloConfig
from ..const import *


class TelemUtil:
    config_file = "/etc/teleport.yaml"

    @staticmethod
    def telem_running():
        return os.path.exists("/run/teleport.pid")

    @staticmethod
    def telem_status():
        from allo.model.colors import BColors
        print("Etat de le télémaintenance : ", end="")
        if TelemUtil.telem_running():
            BColors.success("Démarrée")
        else:
            BColors.error("Stoppée")

    @staticmethod
    def telem_start():
        from allo.utils.ansible import AnsibleUtil
        TelemUtil.__init()
        AnsibleUtil.start_telem()
        TelemUtil.telem_status()

    @staticmethod
    def telem_stop():
        from allo.utils.ansible import AnsibleUtil
        TelemUtil.__init()
        AnsibleUtil.stop_telem()
        TelemUtil.telem_status()

    @staticmethod
    def __init():
        if os.path.exists(TelemUtil.config_file):
            return
        config = AlloConfig.load()
        teleport_yaml = ("teleport:",
                         '    auth_token: "{}"'.format(config.teleport_token),
                         '    ca_pin: "sha256:' + TELEPORT_SHA + '"',
                         '    auth_servers:',
                         '        - {}'.format(ALLO_URL),
                         'auth_service:',
                         '    enabled: no',
                         'proxy_service:',
                         '    enabled: no',
                         'ssh_service:',
                         '    enabled: "yes"',
                         '    labels:',
                         '        produit: {}'.format(config.code_produit),
                         '        secret: {}'.format(config.id_client),
                         '        internal: {}'.format("true" if config.internal else "false"),
                         '    commands:',
                         '    - name: diff',
                         '      command: [allo, changes]',
                         '      period: 1m0s')

        with open(TelemUtil.config_file, 'w+') as yaml_file:
            yaml_file.write('\n'.join(teleport_yaml))



import os
import click
from allo.model.colors import BColors
from allo.model.config import AlloConfig
from allo.telem.telem import TelemUtil
from allo.utils.api import API


def exit_if_no_conf():
    if not hasattr(AlloConfig.load(), "repo_path"):
        BColors.error("Produit non installé ou non détecté.")
        exit(1)


@click.group(invoke_without_command=True)
@click.argument('version', required=False)
@click.option('--beta', is_flag=True)
def update(version, beta):
    """
    Liste les versions disponibles à la mise à jour.

    Si VERSION est renseigné, une mise à jour vers VERSION est déclenchée.

    \b
    VERSION est le numéro de version vers laquelle mettre à jour.
    Si VERSION = "latest", la dernière version disponible sera déployée.
    """
    exit_if_no_conf()
    if version is None:
        from allo.utils.api import API
        print("Versions disponibles :")
        result = API.get_versions(beta)
        for version in result:
            print("- {}".format(version["name"].split(" ")[1]))
    else:
        TelemUtil.telem_start()
        update_to_version(version, beta)


def update_to_version(version, beta):
    result = API.get_versions(beta)
    if len(result) == 0:
        BColors.success("Vous êtes déjà dans la version version disponible")
    else:
        toupdate = None
        if version == "latest":
            toupdate = result[0]
        else:
            for ver in result:
                if ver["name"].split(" ")[1] == version:
                    toupdate = ver

        if toupdate is not None:
            update_request(toupdate, beta)
        else:
            BColors.error("Version indisponible")


def update_request(version, beta):
    if beta:
        BColors.error("ATTENTION: Une mise à jour en version BETA a été demandée.\n"
                      "/!\\ Ce type d'installation est hors support LIBRICIEL")
    BColors.warning("Une mise à jour en version {} va être déclenchée. "
                    "Confirmer ? [o/N] ".format(version["name"].split(" ")[1]))
    confirm = input()
    if confirm.lower() == "o":
        if API.update_to_version(version["id"], beta):
            BColors.info("Mise à jour en version {}...".format(version["name"].split(" ")[1]))

            progress_file = "/tmp/progress.log"
            if os.path.exists(progress_file):
                os.remove(progress_file)

            from allo.utils.logreader import LogReader
            logreader = LogReader(1, progress_file, True)
            logreader.start()
            logreader.join()

        else:
            BColors.error("Erreur lors de la demande de mise à jour")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import random
import platform
from uuid import getnode as get_mac

TELEPORT_SHA = "786e399511525889778d2a3e64bc5d18f7ecddf6eaac684c224249657f404b76" \
    if os.getenv("ALLOENV") == "TEST" \
    else "fa04922fca037e9f2f6ff0c770ab10c746843c91b4aad8008df24d8b79b6b206"

ALLO_INFO_PATH = '/tmp/allo-infos.yml'

CONFIG_PATH = "/etc/allo-config.dict"

ALLO_URL = "10.81.41.1:3025" \
    if os.getenv("ALLOENV") == "TEST" \
    else "teleport.libriciel.fr:443"

API_PATH = "https://{}/v1/webapi".format("10.81.41.1:3080") \
    if os.getenv("ALLOENV") == "TEST" \
    else "https://{}/v1/webapi".format(ALLO_URL)

CODEPRODUIT = {
    "i-Parapheur": "IP",
    "Pastell": "PA",
    "web-delib": "WD",
    "web-DPO": "DP",
}


def getnode():
    random.seed(platform.node() + str(get_mac()))
    return random.randint(1, 999999999999)



import os
import subprocess
import sys

import progressbar


class AnsibleUtil:
    def __init__(self, process, widgets):
        self.process = process
        self.iterations = 0
        self.bar = progressbar.ProgressBar(max_value=progressbar.UnknownLength, widgets=widgets)
        self.__process()

    def __wait(self):
        try:
            self.iterations += 1
            self.bar.update(self.iterations)
            self.process.wait(0.2)
            self.bar.finish()
            return True
        except subprocess.TimeoutExpired:
            return False

    def __process(self):
        while not self.__wait():
            # Do nothing, just wait and update progressbar
            pass
        sys.stdout.write("\033[F")
        sys.stdout.write("\033[K")

    @staticmethod
    def create_user(product):
        AnsibleUtil._run_playbook("create_user.yml", [
            'Création de l\'utilisateur de télémaintenance : ', progressbar.AnimatedMarker()
        ], "user_name=libriciel-{}".format(product.lower()))

    @staticmethod
    def install_dependencies():
        AnsibleUtil._run_playbook("install_allo.yml", [
            'Installation des dépendances : ', progressbar.AnimatedMarker()
        ])

    @staticmethod
    def start_telem():
        AnsibleUtil._run_playbook("start_service.yml", [
            'Lancement de la télémaintenance : ', progressbar.AnimatedMarker()
        ])

    @staticmethod
    def stop_telem():
        AnsibleUtil._run_playbook("stop_service.yml", [
            'Arrêt de la télémaintenance : ', progressbar.AnimatedMarker()
        ])

    @staticmethod
    def _run_playbook(ymlfile, widgets, pb_vars=""):

        fh = open(os.devnull, "w")
        AnsibleUtil(subprocess.Popen(
            (
                'ansible-playbook',
                os.path.dirname(os.path.realpath(__file__)) + "/playbooks/" + ymlfile,
                "--extra-vars",
                pb_vars
            ), stdout=fh, stderr=fh), widgets)

        fh.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .changes import commands as changes
from .telem import commands as telem
from .update import commands as update
from .install import commands as install

import click
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


@click.group(invoke_without_command=True)
@click.version_option()
@click.pass_context
def cmd(ctx):
    """Outil de télémaintenance et de mise à jour automatique Libriciel-SCOP"."""

    from allo.model.config import AlloConfig
    from allo.model.colors import BColors

    if not ctx.invoked_subcommand:
        import allo
        from allo.utils.configloader import ConfigLoader
        from allo.utils.ansible import AnsibleUtil

        BColors.info("ALLO-NG v{} - Utilitaire de mise a jour automatique et telemaintenance".format(allo.__version__))

        AnsibleUtil.install_dependencies()
        ConfigLoader("PROD")

        BColors.success("Initialisation terminée")
    elif not hasattr(AlloConfig.load(), "teleport_token"):
        BColors.error("Allo non initialisé. Merci de lancer la commande 'allo'")
        exit(1)


cmd.add_command(changes.changes)
cmd.add_command(telem.telem)
cmd.add_command(update.update)
cmd.add_command(install.install)



import click
from allo.model.config import AlloConfig
from allo.model.colors import BColors
from .telem import TelemUtil


@click.group(invoke_without_command=True)
@click.pass_context
def telem(ctx):
    """Affiche l'état de la télémaintenance."""
    if not hasattr(AlloConfig.load(), "teleport_token"):
        BColors.error("Allo non initialisé. Merci de lancer la commande 'allo'")
        exit(1)

    if not ctx.invoked_subcommand:
        TelemUtil.telem_status()


@telem.command()
def start():
    """Lancement de la télémaintenance."""
    TelemUtil.telem_start()


@telem.command()
def stop():
    """Arrêt de la télémaintenance."""
    TelemUtil.telem_stop()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import requests
from requests import Response
from .. import const
from ..model.colors import BColors


class API:
    @staticmethod
    def find_instance():
        with open(const.ALLO_INFO_PATH, 'rb') as payload:
            r = requests.post("{}/referees/{}".format(const.API_PATH, const.getnode()), verify=False, data=payload)
            if API.has_error(r):
                return False
            return r.json()

    @staticmethod
    def get_secret():
        r = requests.get("{}/referees/{}/secret".format(const.API_PATH, const.getnode()), verify=False)
        if API.has_error(r):
            return False
        return r.text

    @staticmethod
    def ask_install():
        with open(const.ALLO_INFO_PATH, 'rb') as payload:
            r = requests.post("{}/referees/{}/install".format(const.API_PATH, const.getnode()), verify=False, data=payload)
            if API.has_error(r):
                return False
            return r.text

    @staticmethod
    def get_versions(beta):
        url = "{}/referees/{}/versions/beta" if beta else "{}/referees/{}/versions"
        r = requests.get(url.format(const.API_PATH, const.getnode()), verify=False)
        if API.has_error(r):
            return False
        return r.json()

    @staticmethod
    def update_to_version(versionid, beta):
        with open(const.ALLO_INFO_PATH, 'rb') as payload:
            url = "{}/referees/{}/update/{}/beta" if beta else "{}/referees/{}/update/{}"
            r = requests.post(url.format(const.API_PATH, const.getnode(), versionid),
                              verify=False, data=payload)
            if API.has_error(r):
                return False
            return r.text

    @staticmethod
    def has_error(r: Response):
        if r.status_code is 502:
            BColors.error("Erreur lors de la communication avec Allo Server : Erreur {}".format(r.status_code))
            return True
        return False



# -*- coding: utf-8 -*-

from .channel import Channel


class Version:
    def __init__(self, obj):
        self.id = obj['id']
        self.name = obj['name']
        self.git_path = obj['gitPath']
        self.channel = Channel(obj['channel'])



import click
from allo.model.config import AlloConfig
from allo.model.colors import BColors


def exit_if_no_conf():
    if not hasattr(AlloConfig.load(), "repo_path"):
        BColors.error("Produit non installé ou non détecté.")
        exit(1)


@click.group(invoke_without_command=True)
@click.pass_context
def changes(ctx):
    """Liste des différences dans l'application."""

    if not ctx.invoked_subcommand:
        if not hasattr(AlloConfig.load(), "repo_path"):
            return
        from allo.utils.git import GitHelper
        print(GitHelper().get_diff())


@changes.command(name='init')
@click.argument('path')
@click.argument('message')
def init_git_repo(path, message):
    """Initialisation du dossier d'installation.

    \b
    PATH est le chemin de l'installation.
    MESSAGE est le message d'initialisation.
    """
    from allo.utils.git import GitHelper
    helper = GitHelper()
    helper.set_git_root_directory(path)
    helper.init_repo(message)

    BColors.success("Dossier initialisé")


@changes.command(name='update')
@click.argument('message')
def update_git_reo(message):
    """Modification du dossier d'installation.

    \b
    MESSAGE est le message de modification.
    """
    from allo.utils.git import GitHelper
    exit_if_no_conf()
    GitHelper().update(message)

    BColors.success("Dossier mis à jour")



import threading
import os
import time

import progressbar
from allo.model.colors import BColors


class LogReader(threading.Thread):
    def __init__(self, threadid, filename, progress):
        threading.Thread.__init__(self)
        self.threadID = threadid
        self.filename = filename
        self._stop_event = threading.Event()
        self.progress = progress
        self.bar = None
        self._release = ""
        self.release_mode = False
        self.failed = False

    def stop(self):
        self._stop_event.set()

    def stopped(self):
        return self._stop_event.is_set()

    def follow(self):
        """
        generator function that yields new lines in a file
        """
        # seek the end of the file
        self.logfile.seek(0, os.SEEK_END)

        first_line = True
        # start infinite loop
        while True:
            if self.stopped():
                return
            # read last line of file
            line = self.logfile.readline()
            # sleep if file hasn't been updated
            if not line:
                if self.stopped():
                    return
                if self.bar is not None:
                    self.bar.update(1)
                time.sleep(0.1)
                continue
            if first_line:
                first_line = False
            if not self.have_to_handle_special_line(line):
                yield line

    def have_to_handle_special_line(self, line):
        handled = False
        if self.bar is not None:
            if "RELEASE_END" in line:
                self.release_mode = False
                handled = True
            elif "RELEASE" in line:
                self.release_mode = True
                handled = True
            elif "RESCUE" in line:
                self.bar.finish(end=BColors.error(" Erreur de mise à jour. Restauration en cours...\n", False))
                self.failed = True
                handled = True
            elif "FAILED" in line:
                self.bar.finish(end=BColors.error(" ERREUR CRITIQUE\n", False))
                self.failed = True
                handled = True
            elif self.release_mode:
                self._release = self._release + line
                handled = True
            else:
                self.bar.finish(end=BColors.success(" OK\n", False))
                if "ENDED" in line:
                    BColors.success("Mise à jour terminée")
                    self.stop()
                    handled = True
        return handled

    def run(self):
        while not os.path.exists(self.filename):
            if self.stopped():
                return
            time.sleep(1)

        self.logfile = open(self.filename, "r")
        loglines = self.follow()
        # iterate over the generator
        for line in loglines:
            if self.progress:
                self.bar = progressbar.ProgressBar(max_value=progressbar.UnknownLength, widgets=[
                    line.rstrip() + " ", progressbar.AnimatedMarker()
                ])
            else:
                print(line, end="")

        if not self.failed and self._release:
            BColors.info("Note de version - Informations supplémentaires: ")
            print(self._release)



# -*- coding: utf-8 -*-
import pickle
import os

from allo.const import CONFIG_PATH


class AlloConfig:
    internal = False

    @staticmethod
    def load():
        if os.path.exists(CONFIG_PATH):
            with open(CONFIG_PATH, 'rb') as config_dictionary_file:
                return pickle.load(config_dictionary_file)
        else:
            return AlloConfig()

    def save(self):
        with open(CONFIG_PATH, 'wb') as config_dictionary_file:
            pickle.dump(self, config_dictionary_file)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import allo

setup(
    name='allo-client.beta',
    version=allo.__version__,
    author="Lukas Hameury",
    author_email="lukas.hameury@libriciel.coop",
    description="Libriciel upgrade package",
    long_description=open('README.md', encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://gitlab.libriciel.fr/libriciel/projets-internes/allo/allo-client",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'PyInquirer',
        'requests>=2.16.0',
        'gitpython',
        'PyYAML',
        'progressbar2',
        'jsons',
        'ansible',
        'click',
        'urllib3'
    ],
    entry_points={
        'console_scripts': [
            'allo = allo.cmd:cmd'
        ],
    },
)



# -*- coding: utf-8 -*-
from time import sleep

from PyInquirer import prompt

from .ansible import AnsibleUtil
from .. import const
from ..model.colors import BColors
from ..model.config import AlloConfig
from ..utils.api import API


class ConfigLoader:
    def __init__(self, env):
        self.config = AlloConfig.load()
        # Check we have (id_client OR internal) AND teleport_token
        if (hasattr(self.config, "id_client") or self.config.internal) and hasattr(self.config, "teleport_token"):
            self.found_instance = API.find_instance()
        else:
            # We have to load the self.config from association
            self.config.env = env
            self.found_instance = API.find_instance()
            self.secret = API.get_secret()
            # We handle the found state and save the config file
            self.handle_state()
            # Create specific user to access node only if product is well known
            if self.config.code_produit in const.CODEPRODUIT.values():
                AnsibleUtil.create_user(self.config.code_produit.lower())
        self.__show_instance_informations()

    def __show_instance_informations(self):
        if "server_node_c" not in self.found_instance:
            BColors.header("Instance interne détectée")
        else:
            account_name = self.found_instance["account"]["name"]\
                if self.found_instance["account"] is not None and "name" in self.found_instance["account"] \
                else "Aucun compte associé"
            BColors.header("- Compte              : {}".format(account_name) + "\n" +
                           "- Type d'exploiration : {}".format(self.found_instance["type_exploitation"]) + "\n" +
                           "- Produit             : {}".format(self.found_instance["produit"]) + "\n" +
                           "- Version             : {}".format(self.found_instance["version_actuelle"]))

    def __association_handle(self):
        if "message" in self.found_instance and self.found_instance["message"] == "instance not found":
            BColors.info("En attente d'association... Identifiant du noeud : " + self.secret)
            while "token" not in self.found_instance:
                sleep(5)
                self.found_instance = API.find_instance()
            BColors.success("Association effectuée avec succès")

    def __product_handle(self):
        if "server_node_c" in self.found_instance:
            self.config.id_client = self.found_instance["server_node_c"]
            self.config.code_produit = const.CODEPRODUIT[self.found_instance["produit"]]
        else:
            self.config.id_client = self.secret
            self.config.internal = True
            self.config.code_produit = prompt(
                [{'type': 'input', 'name': 'code_produit', 'message': 'Code produit'}])['code_produit']

    def __token_handle(self):
        if "token" in self.found_instance:
            self.config.teleport_token = self.found_instance["token"]
        else:
            self.config.teleport_token = prompt(
                [{'type': 'input', 'name': 'teleport_token', 'message': 'Token de télémaintenance'}])['teleport_token']

    def handle_state(self):
        self.__association_handle()
        self.__product_handle()
        self.__token_handle()
        self.config.save()



from allo.model.colors import BColors
from git import Repo, InvalidGitRepositoryError

from allo.model.config import AlloConfig


class GitHelper:
    """
    Utilitaire GIT pour allo, ajoutant une couche d'abstraction
    """
    def __init__(self):
        self.allo_config = AlloConfig.load()
        if hasattr(self.allo_config, "repo_path"):
            self.set_git_root_directory(self.allo_config.repo_path)

    def set_git_root_directory(self, path: str):
        # Set repo path and save config
        self.allo_config.repo_path = path
        self.allo_config.save()

    def init_repo(self, message: str):
        if not self.__load_repo(False):
            self.git_repo = Repo.init(self.allo_config.repo_path)
            self.git_repo.git.add(all=True)
            self.git_repo.index.commit(message)

    def get_diff(self):
        if not self.__load_repo():
            return ""
        head = self.git_repo.head.commit.tree
        return self.git_repo.git.diff(head).rstrip()

    def update(self, message: str):
        if not self.__load_repo():
            return
        self.git_repo.git.add(all=True)
        self.git_repo.index.commit(message)

    def __load_repo(self, print_error: bool = True):
        try:
            self.git_repo = Repo(self.allo_config.repo_path)
        except InvalidGitRepositoryError:
            if print_error:
                BColors.warning("Répertoire non initialisé")
            return False
        return True





import os
import time
import click
from allo.model.colors import BColors

from allo.model.config import AlloConfig


def exit_if_no_conf():
    if not hasattr(AlloConfig.load(), "repo_path"):
        BColors.error("Produit non installé ou non détecté.")
        exit(1)

@click.command()
def install():
    """Demande d'installation du produit."""
    from allo.telem.telem import TelemUtil
    from allo.utils.api import API
    from allo.utils.ansible import AnsibleUtil
    from allo.utils.logreader import LogReader

    # Install deps and init allo yml file
    AnsibleUtil.install_dependencies()
    TelemUtil.telem_start()
    # Wait 2 seconds, waiting for node to be visible on teleport side
    time.sleep(2)

    fileresult = "/tmp/install_result"

    if os.path.exists(fileresult):
        os.remove(fileresult)
    API.ask_install()

    logreader = LogReader(1, "/tmp/install.log", False)
    logreader.start()

    print("En attente de lancement de l'installation ...")

    while not os.path.exists(fileresult):
        time.sleep(1)

    print("Installation terminée")
    logreader.stop()
    logreader.join()
